/*
   prog5-?.cpp

   En enkel strngklass som demonstrerar verlagrade operatorer

   Klassen cstring (inte string, fr att undvika konflikt med  
   eventuella strngklasser i kompilatorns bibliotek) inkapslar
   vanliga teckenstrngar och tar hand om den allokering och de-
   allokering av minne som krvs. Operatorer fr tilldelning,
   tkomst av enskilda tecken, jmfrelse och sammanslagning har
   verlagrats.  
   Fr att inte krngla till exemplet kopieras strngdata alltid,
   och inga felkontroller grs. Jmfrelse av strngar sker med
   strcmp, vilket kan ge ovntat resultat (tex fr strngar inne-
   hllande ,  och ). 

*/

#include <stdio.h>
#include <string.h>   // <-- behvs fr standardstrngfunktionerna

class cstring
{
   private:

     // pekare till strngen
     char* ptr;

   public:

     // frvald konstruktor
     cstring() { ptr = 0; }

     // konstruktor fr initiering med strngpekare
     cstring(const char*);

     // kopieringskonstruktor
     cstring(const cstring&);

     // destruktor
     ~cstring();

     // skriv ut strngen
     void print() { printf("\"%s\"",ptr); }

     // verlagrade tilldelningsoperatorer
     cstring& operator=(const char*);
     cstring& operator=(const cstring&);

     // verlagrade operatorer fr tkomst till vektorelement
     // notera att vi kan verlagra medlemmar som r const
     // respektive icke-const
     char& operator[](int i)
        { return ptr[i]; }
     const char& operator[](int i) const 
        { return ptr[i]; }

     // verlagrade relationsoperatorer
     int operator==(const char* s) const
        { return strcmp(ptr,s) == 0; }
     int operator==(const cstring& s) const
        { return strcmp(ptr,s.ptr) == 0; }
     int operator!=(const char* s) const
        { return strcmp(ptr,s) != 0; }
     int operator!=(const cstring& s) const
        { return strcmp(ptr,s.ptr) != 0; }
     int operator<(const char* s) const
        { return strcmp(ptr,s) < 0; }
     int operator<(const cstring& s) const
        { return strcmp(ptr,s.ptr) < 0; }
     int operator>(const char* s) const
        { return strcmp(ptr,s) > 0; }
     int operator>(const cstring& s) const
        { return strcmp(ptr,s.ptr) > 0; }
     int operator<=(const char* s) const
        { return strcmp(ptr,s) <= 0; }
     int operator<=(const cstring& s) const
        { return strcmp(ptr,s.ptr) <= 0; }
     int operator>=(const char* s) const
        { return strcmp(ptr,s) >= 0; }
     int operator>=(const cstring& s) const
        { return strcmp(ptr,s.ptr) >= 0; }

     // verlagrad additionsoperator 
     friend cstring operator+(const cstring& s1,const cstring& s2);
};

cstring::cstring(const char* s)
{
   ptr = new char[strlen(s)+1];
   strcpy(ptr,s);
}

cstring::cstring(const cstring& s)
{
   ptr = new char[strlen(s.ptr)+1];
   strcpy(ptr,s.ptr);
}

cstring::~cstring()
{
   if(ptr)
   {
      delete[] ptr;
   }
}

cstring& cstring::operator=(const char* s)
{
   if(ptr)
   {
      delete[] ptr;
   }
   ptr = new char[strlen(s)+1];
   strcpy(ptr,s);
   return *this;
}

cstring& cstring::operator=(const cstring& s)
{
   if(ptr)
   {
      delete[] ptr;
   }
   ptr = new char[strlen(s.ptr)+1];
   strcpy(ptr,s.ptr);
   return *this;
}

// verlagrad additionsoperator - lgger ihop tv strngar.
// Notera att denna operator inte r medlem i cstring, bara
// en vn till klassen.
cstring operator+(const cstring& s1,const cstring& s2)
{
   cstring s;

   s.ptr = new char[strlen(s1.ptr)+strlen(s2.ptr)+1];
   strcpy(s.ptr,s1.ptr);
   strcat(s.ptr,s2.ptr);
   return s;
} 

// testa strngklassen
main()
{
   // initiering
   cstring s  = "musselsoppa";
   cstring s2 = s;
   cstring s3 = s + ", tack!";
   cstring s4;

   // tilldelning och addition
   s4 = s2 + "ns at" + "tack!";

   // utskrift
   printf("s  = "); s.print(); 
   printf("\ns2 = "); s2.print(); 
   printf("\ns3 = "); s3.print(); 
   printf("\ns4 = "); s4.print(); 

   // jmfrelse
   if(s == s2)
   {
      printf("\ns == s2\n");
   }
   if(s2 != s3)
   {
      printf("s2 != s3\n");
   }
   if(s3 < s4)
   {
      printf("s3 < s4\n");
   }

   // tkomst till enskilda tecken
   printf("s[1] = '%c'\n",s[1]);

   return 0;
}
